#!/usr/bin/python3

#	control++ : System configuration tool
#	Copyright (C) 2017-2021 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import os
from shutil import rmtree

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

class TestEnv():

	def __init__(self, test_dir, test_files, exec_dir, recreate=False,
			printer=None):

		self.test_dir = test_dir
		self.test_files = test_files
		self.exec_dir = exec_dir
		self.printer  = printer
		self.recreate = recreate

	# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

	def Prepare(self):

		# Creating the test dir itself
		if not self.__CreateDir(self.test_dir):
			return False

		# Creating the files of the test file structure
		testdir_level = len(self.test_dir.split('/'))
		for path in self.test_files:
			files = path.split('/')
			for level in range(testdir_level, len(files)):
				subpath = '/'.join(files[:level + 1])
				if os.path.exists(subpath):
					continue
				self.__Print(f'Creating "{subpath}"')
				if os.path.normpath(subpath)[-2] != 'f':
					os.mkdir(subpath)
				else:
					os.mknod(subpath, mode = 0o777)

		return True

	# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

	def Clear(self):

		pycache_dir = os.path.join(self.exec_dir, '__pycache__')

		return \
			self.__ClearDir(self.test_dir) and \
			self.__ClearDir(pycache_dir, rmdir=True)

	# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

	def __ClearDir(self, dir_name, rmdir=False):

		if not os.path.isdir(dir_name):
			self.__Print(f'"{dir_name}" is not a directory')
			return False

		for file_name in os.listdir(dir_name):
			full_file_path = os.path.join(dir_name, file_name)
			try:
				if os.path.isfile(full_file_path):
					os.remove(full_file_path)
				elif os.path.isdir(full_file_path):
					rmtree(full_file_path)
			except Exception as e:
				self.__Print(str(e))
				return False

		if rmdir:
			try:
				os.rmdir(dir_name)
			except Exception as e:
				self.__Print(str(e))
				return False

		return True

	# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

	def __Print(self, message):

		if self.printer:
			self.printer.LineEnd('\t' + message)

	# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

	def __CreateDir(self, path):

		if not path:
			self.__Print('Empty path')
			return False

		EXISTS = os.path.exists(path)
		IS_DIR = os.path.isdir(path)

		if EXISTS and not IS_DIR:
			self.__Print(f'Can\'t create "{path}" - there is a file '
				'with a same name')
			return False
		if EXISTS and IS_DIR:
			if self.recreate:
				if not self.__ClearDir(path):
					self.__Print('Can\'t recreate ' + path)
					return False
			else:
				self.__Print(path + ' is already exist')
				return True
		elif not EXISTS:
			self.__Print('Creating ' + path)
			os.makedirs(path)

		return True
